import socket, string, sys

# YHeroic is class/library for communicatio with HeaRT engine via TCP protocol.
# @author Szymon Bobek
# @author Michal Gawedzki
# @version 1.0.1
# @license GPL 3
#
# 28.06.2009
#
class YHeroic:

	# Constructor, init intance of PHeroic library
	# @argument hostanem: hostname where HeaRT server can be found
	# @argument port: port where HeaRT server is available
	# @return: instance of object 
	def __init__(self, hostname, port):
		self.hostname = hostname
		self.port = port
		self.protocolVersion = "1.0"
		print(self.protocolVersion)
		

	# Returns list of available models [user,model]
	# @return: list of available models
	def getModelList(self):
		request =  '[model,getlist].\n'
		response = self.performRequest(request)
		return response

	# Allows to get user's specified model
	# @argument userName: name of model's owner
	# @argument modelName: name of model you want to get
	# @return: user model as string
	def getUserModel(self, userName, modelName):
		request = "[model,get,'" + modelName + "','" + userName + "'].\n"
		response = self.performRequest(request)
		return response
	
	# Allows to add new model as specified user.
	# @argument userName: name of model's owner
	# @argument modelName: name of model you want to add
	# @argument model: model as string
	# @return: request status
	#
	# IMPORTANT! Model in last argument must be: one line, without " characters and with end line char \n changed to \\n (means \n must be seen)	
	def addUserModel(self, userName, modelName, model):
		model = model.replace("'","\\'")
		request = "[model,add,'" + modelName + "','" + userName + "','" + model + "'].\n"
		response = self.performRequest(request)
		return response

	# Allows to remove user specified model
	# @argument userName: name of model's owner
	# @argument modelName: name of model to remove
	# @return: request status
	def removeUserModel(self, userName, modelName):
		request = "[model,remove,'" + modelName + "','" + userName + "'].\n"
		response = self.performRequest(request)
		return response

	# Starts simulation of selected model
	# @argument userName: name of model's owner
	# @argument modelName: name of model you want to simulate
	# @argument infType: type of inference. "ddi" = Data Driven, "gdi" = Goal Driven, "tdi" = Token Driven
	# @argument tables: list of tables, separate with , char
	def runInference(self, userName, modelName, infType, tables, state):
		inference = ""
		if infType == "ddi":
			inference = "dd"
		elif infType == "gdi":
			inference = "gdi"
		elif infType == "tdi":
			inference = "td"
		else:
			inference = "dd"

		request = "[model,run,'" + modelName+ "','" + userName + "','" + inference + "','[" + tables + "]','" + state + "'].\n"
		response = self.performRequest(request)
		return response

	# Adds new state to selected model
	# @argument userName: name of model's owner
	# @argument modelName: name of the model
	# @argument stateName: name of the state you want to add
	# @argument stateDef: definition of the state
	# @return: request status
	def addStateToModel(self, userName, modelName, stateName, stateDef):
		request = "[state,add,'" + modelName + "','" + userName + "','" + stateName + "','" + stateDef + "'].\n"
		response = self.performRequest(request)
		return response

	# Removes state from selected model
	# @argument userName: name of model's owner
	# @argument modelName: name of the model
	# @argument stateName: name of state that you want to remove
	# @return: request status
	def removeStateFromModel(self, userName, modelName, stateName):
		request = "[stste,remove,'" + modelName + "','" + userName + "','" + stateName + "'].\n"
		response = self.performRequest(request)
		return response

	# Returns current protocol version
	# @return: protocol version as string
	def getProtocolVersion(self):
		return self.protocolVersion

	# Runs model's verification
	# @argument userName: name of model's owner
	# @argument modelName: name of the model
	# @argument mode: mode of verification, "comp" = complete (vcomplete), "contr" = contradict (vcontradict), "subs" = subsume (vsubsume), "redu" = reduce (vreduce)
	# @return: verification status
	def verifyModel(self, userName, modelName, mode):
		verification = ""
		
		if mode == "comp":
			verification = "vcomplete"
		elif mode == "contr":
			verification = "vcontradict"
		elif mode == "subs":
			verification = "vsubsume"
		elif mode == "redu":
			verification = "vreduce"

		request = "[model,verify,'" + verification + "','" + modelName + "','" + userName + "'].\n"
		response = self.performRequest(request)
		return response

	# Internal communication function, do not use it
	def performRequest(self, request):
		s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		s.connect((self.hostname, self.port))
		sent = s.send(request)

		data = s.recv(1024)
		response = ""
		while len(data):
			response = response + data
			data = s.recv(1024)

		s.close()
		return response

